import os

from unittest import TestCase
from pymuxinator import config

class FixturedTestCase(TestCase):
    def __init__(self, *args, **kwargs):
        config.root = os.path.dirname(__file__) + '/fixtures/'

        super(FixturedTestCase, self).__init__(*args, **kwargs)



import mock

from jinja2 import Template
from pymuxinator import config, exceptions
from tests import FixturedTestCase
from unittest import TestCase


class TestListAll(FixturedTestCase):
    def test_list(self):
        configs = config.list_all()

        # strip root directory out of path names
        configs = [c.replace(config.root, '') for c in configs]

        # make sure it walks the file tree
        self.assertIn('sample.yml', configs)
        self.assertIn('level1/one.yml', configs)
        self.assertIn('level1/level2/two.yml', configs)

    def test_bad_root_path(self):
        # mock config
        old_value = config.root
        config.root = 'dasfjaskldaj'

        # make sure it doesn't error
        self.assertEqual(config.list_all(), [])

        # reset config
        config.root = old_value


class TestProjectPath(FixturedTestCase):
    def test_existing_path(self):
        expected = config.root + 'sample.yml'
        self.assertEqual(config.project_path('sample'), expected)

    def test_non_existing_path(self):
        with self.assertRaises(exceptions.ProjectNotFoundError):
            config.project_path('foobarrr')


class TestLoad(FixturedTestCase):
    def test_load(self):
        """Sanity check that it loads the YAML from the file"""
        sample = config.load('sample')
        self.assertEqual(sample['root'], '~/test')

    def test_load_parse_error(self):
        with self.assertRaises(exceptions.ConfigParseError):
            config.load('bad_syntax')


class TestLoadTemplate(TestCase):
    def test_load(self):
        """Make sure the template is loaded correctly"""
        template = config.load_template('tmux')
        self.assertTrue(isinstance(template, Template))


class TestGetEnvVar(TestCase):
    @mock.patch('os.environ.get', return_value='foo')
    def test_get(self, mock_get):
        self.assertEqual(config.get_env_var('HOME'), 'foo')

    @mock.patch('os.environ.get')
    def test_get_default(self, mock_get):
        config.get_env_var('HOME', 'testing')
        mock_get.assert_called_once_with('HOME', 'testing')



import mock

from pymuxinator import utils
from unittest import TestCase


class TestShellEscape(TestCase):
    def test_normal_string(self):
        self.assertEqual(utils.shell_escape('foo'), 'foo')

    def test_escape(self):
        self.assertEqual(utils.shell_escape('"foo"'), '\'"foo"\'')


class TestExpandPath(TestCase):
    @mock.patch('os.path.expanduser', return_value='foo')
    def test_user_path(self, mock_expanduser):
        self.assertEqual(utils.expand_path('~/foo'), 'foo')

    def test_normal_path(self):
        self.assertEqual(utils.expand_path('/home//dir/./foo'), '/home/dir/foo')


class TestExecuteCmd(TestCase):
    def test_execute(self):
        self.assertEqual(utils.execute_cmd('echo "HEY"'), 'HEY\n')



import mock

from pymuxinator.pane import Pane
from pymuxinator.project import Project
from pymuxinator.window import Window
from tests import FixturedTestCase


class WindowTestCase(FixturedTestCase):
    window_idx = 0

    def setUp(self):
        self.project = Project.from_name('sample')
        self.window = self.project.windows[self.window_idx]


class TestWindow(WindowTestCase):
    def test_name(self):
        self.assertEqual(self.window.name, 'editor')

    def test_layout(self):
        self.assertEqual(self.window.layout, 'main-vertical')

    def test_pre(self):
        expected = 'echo "no pane, no gain!"'
        self.assertEqual(self.window.pre, expected)

    def test_pane_count(self):
        self.assertEqual(self.window.pane_count, 4)

    def test_panes(self):
        panes = self.window.panes
        self.assertEqual(len(panes), 4)

        for idx, pane in enumerate(panes):
            self.assertTrue(isinstance(pane, Pane))
            self.assertEqual(pane.index, idx)
            self.assertTrue(len(pane.commands) > 0)
            self.assertTrue(pane.window_config, self.window.pane_config)

    @mock.patch('pymuxinator.window.Window.pre',
        return_value='fake_pre', new_callable=mock.PropertyMock)
    @mock.patch('pymuxinator.window.Window.pane_count',
        return_value=55, new_callable=mock.PropertyMock)
    @mock.patch('pymuxinator.window.Window.tmux_window_target',
        return_value='fake_target', new_callable=mock.PropertyMock)
    def test_pane_config(self, mock_target, mock_pc, mock_pre):
        self.assertEqual(self.window.pane_config, {
            'index': 0,
            'pre': 'fake_pre',
            'pane_count': 55,
            'tmux_window_target': 'fake_target',
        })

    def test_commands(self):
        self.assertEqual(self.window.commands, None)

    def test_tmux_window_target(self):
        self.assertEqual(self.window.tmux_window_target, 'sample:0')

    @mock.patch('pymuxinator.window.Window.tmux_window_target',
        return_value='fake_target', new_callable=mock.PropertyMock)
    def test_tmux_pre_window_command(self, mock_target):
        self.window.project_config['tmux'] = 'fakemux'
        self.window.project_config['pre_window'] = 'fake_pre'

        expected = 'fakemux send-keys -t fake_target fake_pre C-m'
        self.assertEqual(self.window.tmux_pre_window_command, expected)

    def test_tmux_window_command_prefix(self):
        self.window.project_config['tmux'] = 'fakemux'
        self.window.project_config['name'] = 'fake_name'

        expected = 'fakemux send-keys -t fake_name:0'
        self.assertEqual(self.window.tmux_window_command_prefix, expected)

    @mock.patch('pymuxinator.window.Window.tmux_window_target',
        return_value='fake_target', new_callable=mock.PropertyMock)
    def test_tmux_new_window_command(self, mock_target):
        self.window.project_config['tmux'] = 'fakemux'
        self.window.project_config['root'] = '/fake/root'

        expected = 'fakemux new-window -c /fake/root -t fake_target -n editor'
        self.assertEqual(self.window.tmux_new_window_command, expected)

    @mock.patch('pymuxinator.window.Window.tmux_window_target',
        return_value='fake_target', new_callable=mock.PropertyMock)
    def test_tmux_layout_command(self, mock_target):
        self.window.project_config['tmux'] = 'fakemux'

        expected = 'fakemux select-layout -t fake_target main-vertical'
        self.assertEqual(self.window.tmux_layout_command, expected)

    @mock.patch('pymuxinator.window.Window.tmux_window_target',
        return_value='fake_target', new_callable=mock.PropertyMock)
    def test_tmux_select_first_pane(self, mock_target):
        self.window.project_config['tmux'] = 'fakemux'

        expected = 'fakemux select-pane -t fake_target.0'
        self.assertEqual(self.window.tmux_select_first_pane, expected)


class TestWindowMultiPre(WindowTestCase):
    window_idx = 2

    def test_name(self):
        self.assertEqual(self.window.name, 'split_pane')

    def test_layout(self):
        self.assertEqual(self.window.layout, 'main-horizontal')

    def test_pre(self):
        expected = 'echo "first" && echo "second"'
        self.assertEqual(self.window.pre, expected)

    def test_tmux_window_target(self):
        self.assertEqual(self.window.tmux_window_target, 'sample:2')


class TestWindowListCommands(WindowTestCase):
    window_idx = 1

    def test_commands(self):
        self.assertEqual(self.window.commands, [
            "tmux -f ~/.tmux.testing.conf -L foo send-keys -t sample:1 'git branch' C-m",
            "tmux -f ~/.tmux.testing.conf -L foo send-keys -t sample:1 'git status' C-m",
        ])


class TestWindowStringCommands(WindowTestCase):
    window_idx = 3

    def test_commands(self):
        self.assertEqual(self.window.commands, [
            "tmux -f ~/.tmux.testing.conf -L foo send-keys -t sample:3 'service start wobscale' C-m",
        ])


class TestWindowEmptyStringCommands(WindowTestCase):
    window_idx = 7

    def test_commands(self):
        self.assertEqual(self.window.commands, [])


class TestWindowWithBaseIndex(WindowTestCase):
    window_idx = 1

    def setUp(self):
        super(TestWindowWithBaseIndex, self).setUp()
        self.window.project_config['base_index'] = 55

    def test_tmux_window_target(self):
        self.assertEqual(self.window.tmux_window_target, 'sample:56')

    def test_tmux_window_command_prefix(self):
        self.window.project_config['tmux'] = 'fakemux'
        self.window.project_config['name'] = 'fake_name'

        expected = 'fakemux send-keys -t fake_name:56'
        self.assertEqual(self.window.tmux_window_command_prefix, expected)

    @mock.patch('pymuxinator.window.Window.tmux_window_target',
        return_value='fake_target', new_callable=mock.PropertyMock)
    def test_tmux_select_first_pane(self, mock_target):
        self.window.project_config['tmux'] = 'fakemux'

        expected = 'fakemux select-pane -t fake_target.55'
        self.assertEqual(self.window.tmux_select_first_pane, expected)


from pymuxinator import utils
from pymuxinator.pane import Pane

class Window(object):
    def __init__(self, config, project_config, index):
        self.config = config
        self.project_config = project_config
        self.index = index

        # readonly caches
        self._layout = None
        self._pre = None
        self._pane_count = 0
        self._panes = None
        self._commands = None

        self._build_window()

    @property
    def name(self):
        keys = self.config.keys()
        return len(keys) and utils.shell_escape(keys[0])

    @property
    def layout(self):
        return self._layout

    @property
    def pre(self):
        pre = self._pre

        if isinstance(pre, list):
            pre = [x for x in pre if x]
            pre = ' && '.join(pre)

        return pre or ''

    @property
    def pane_count(self):
        return self._pane_count

    @property
    def panes(self):
        return self._panes

    @property
    def pane_config(self):
        return {
            'index': self.index,
            'pre': self.pre,
            'pane_count': self.pane_count,
            'tmux_window_target': self.tmux_window_target,
        }

    @property
    def commands(self):
        return self._commands

    @property
    def tmux_window_target(self):
        name = self._get_project_setting('name')
        base_index = self._get_project_setting('base_index')

        return u"%s:%i" % (name, self.index + base_index)

    @property
    def tmux_pre_window_command(self):
        pre_window = self._get_project_setting('pre_window')

        if pre_window:
            tmux = self._get_project_setting('tmux')
            window_target = self.tmux_window_target

            pre_window = utils.shell_escape(pre_window)
            pre_window = '%s send-keys -t %s %s C-m' % (tmux, window_target, pre_window,)

        return pre_window or ''

    @property
    def tmux_window_command_prefix(self):
        tmux = self._get_project_setting('tmux')
        name = self._get_project_setting('name')
        index = self.index + self._get_project_setting('base_index')

        return u'%s send-keys -t %s:%i' % (tmux, name, index)

    @property
    def tmux_new_window_command(self):
        tmux = self._get_project_setting('tmux')
        window_target = self.tmux_window_target
        path = self._get_project_setting('root')

        if path:
            path = u'-c %s' % path

        return u'%s new-window %s -t %s -n %s' % (tmux, path, window_target, self.name,)

    @property
    def tmux_layout_command(self):
        tmux = self._get_project_setting('tmux')
        window_target = self.tmux_window_target

        return '%s select-layout -t %s %s' % (tmux, window_target, self.layout,)

    @property
    def tmux_select_first_pane(self):
        tmux = self._get_project_setting('tmux')
        window_target = self.tmux_window_target
        index = self._get_project_setting('base_index') # + self.panes[0].index

        return '%s select-pane -t %s.%i' % (tmux, window_target, index)

    def _build_window(self):
        window_value = self.config.values()[0]

        if isinstance(window_value, dict):
            layout = window_value.get('layout')
            self._layout = layout and utils.shell_escape(layout)

            self._pre = window_value.get('pre')
            self._pane_count = len(window_value.get('panes'))
            self._panes = self._build_panes(window_value.get('panes'))
        else:
            self._commands = self._build_commands(window_value)
            pass

    def _build_panes(self, pane_config):
        panes = []

        for idx, pane in enumerate(pane_config):
            if isinstance(pane, dict):
                commands = pane.values()[0]
            else:
                commands = [pane]

            panes.append(Pane(commands, self.project_config, self.pane_config, idx))

        return panes

    def _build_commands(self, window_value):
        pref = self.tmux_window_command_prefix

        if isinstance(window_value, list):
            return [
                '%s %s C-m' % (pref, utils.shell_escape(command))
                for command in window_value if command
            ]

        if isinstance(window_value, str) and window_value:
            return ['%s %s C-m' % (pref, utils.shell_escape(window_value))]

        return []

    def _get_project_setting(self, key, default=None):
        return self.project_config.get(key, default)



import pipes
import os
import subprocess

def shell_escape(string):
    return pipes.quote(string)

def expand_path(path):
    if path[0] == '~':
        return os.path.expanduser(path)
    else:
        return os.path.abspath(path)

def execute_cmd(cmd):
    return subprocess.check_output(cmd, shell=True)



import os
import fnmatch
import yaml

from jinja2 import Template
from pymuxinator import exceptions

root = os.path.expanduser("~") + u'/.pymuxinator'


def list_all():
    matches = []

    for l_root, dirnames, filenames in os.walk(root):
      for filename in fnmatch.filter(filenames, u'*.yml'):
          matches.append(os.path.join(l_root, filename))

    return matches


def project_path(project_name):
    project_name = project_name + u'.yml'

    for project in list_all():
        if project_name in project:
            return project

    raise exceptions.ProjectNotFoundError


def load(project_name):
    path = project_path(project_name)

    with open(path, 'r') as config_file:
        try:
            return yaml.load(config_file)
        except Exception as e:
            raise exceptions.ConfigParseError(path)

def load_template(template_name):
    path = '%s/templates/%s.template' % (os.path.dirname(__file__), template_name)

    with open(path) as template:
        return Template(template.read())

    return template_name

def get_env_var(var_name, default=None):
    return os.environ.get(var_name, default)



from pymuxinator import utils

class Pane(object):
    def __init__(self, commands, project_config, window_config, index):
        self.commands = commands or []
        self.project_config = project_config
        self.window_config = window_config
        self.index = index

    @property
    def tmux_window_and_pane_target(self):
        project_name = self._get_project_setting('name')
        base_index = self._get_project_setting('base_index')
        window_index = self._get_window_setting('index') + base_index
        pane_index = self.index + base_index

        return u'%s:%i.%i' % (project_name, window_index, pane_index)

    @property
    def tmux_pre_command(self):
        window_pre = self._get_window_setting('pre')

        if window_pre:
            tmux = self._get_project_setting('tmux')
            target = self.tmux_window_and_pane_target

            window_pre = utils.shell_escape(window_pre)
            window_pre = '%s send-keys -t %s %s C-m' % (tmux, target, window_pre,)

        return window_pre or ''

    @property
    def tmux_pre_window_command(self):
        pre = self._get_project_setting('pre_window')

        if pre:
            tmux = self._get_project_setting('tmux')
            target = self.tmux_window_and_pane_target

            pre = utils.shell_escape(pre)
            pre = '%s send-keys -t %s %s C-m' % (tmux, target, pre,)

        return pre or ''

    def tmux_main_command(self, command):
        tmux = self._get_project_setting('tmux')
        project_name = self._get_project_setting('name')

        if command:
            tmux = self._get_project_setting('tmux')
            target = self.tmux_window_and_pane_target

            command = utils.shell_escape(command)
            command = '%s send-keys -t %s %s C-m' % (tmux, target, command,)

        return command or ''

    @property
    def tmux_split_command(self):
        tmux = self._get_project_setting('tmux')
        target = self._get_window_setting('tmux_window_target')

        return '%s splitw -t %s' % (tmux, target,)

    @property
    def is_last(self):
        return self.index == self._get_window_setting('pane_count') - 1

    @property
    def has_multiple_commands(self):
        return self.commands and len(self.commands) > 1

    def _get_project_setting(self, key, default=None):
        return self.project_config.get(key, default)

    def _get_window_setting(self, key, default=None):
        return self.window_config.get(key, default)



import mock

from pymuxinator.project import Project
from pymuxinator.window import Window
from tests import FixturedTestCase


class PaneTestCase(FixturedTestCase):
    pane_idx = 0

    def setUp(self):
        self.project = Project.from_name('sample')
        self.window = self.project.windows[0]
        self.pane = self.window.panes[self.pane_idx]


class TestPane(PaneTestCase):
    def test_has_multiple_commands(self):
        self.assertFalse(self.pane.has_multiple_commands)

    def test_tmux_window_and_pane_target(self):
        self.assertEqual(self.pane.tmux_window_and_pane_target, 'sample:0.0')

    @mock.patch('pymuxinator.pane.Pane.tmux_window_and_pane_target',
        return_value='fake_target', new_callable=mock.PropertyMock)
    def test_tmux_pre_command(self, mock_target):
        self.pane.project_config['tmux'] = 'fakemux'
        self.pane.window_config['pre'] = 'fake_pre'

        expected = 'fakemux send-keys -t fake_target fake_pre C-m'
        self.assertEqual(self.pane.tmux_pre_command, expected)

    @mock.patch('pymuxinator.pane.Pane.tmux_window_and_pane_target',
        return_value='fake_target', new_callable=mock.PropertyMock)
    def test_tmux_pre_window_command(self, mock_target):
        self.pane.project_config['tmux'] = 'fakemux'
        self.pane.project_config['pre_window'] = 'fake_pre_window'

        expected = 'fakemux send-keys -t fake_target fake_pre_window C-m'
        self.assertEqual(self.pane.tmux_pre_window_command, expected)

    @mock.patch('pymuxinator.pane.Pane.tmux_window_and_pane_target',
        return_value='fake_target', new_callable=mock.PropertyMock)
    def test_tmux_main_command(self, mock_target):
        self.pane.project_config['tmux'] = 'fakemux'

        expected = 'fakemux send-keys -t fake_target hey C-m'
        self.assertEqual(self.pane.tmux_main_command('hey'), expected)

    def test_tmux_split_command(self):
        self.pane.project_config['tmux'] = 'fakemux'
        self.pane.window_config['tmux_window_target'] = 'fake_tar'

        expected = 'fakemux splitw -t fake_tar'
        self.assertEqual(self.pane.tmux_split_command, expected)

    def test_is_last(self):
        self.assertFalse(self.pane.is_last)

class TestPaneMultipleCommands(PaneTestCase):
    pane_idx = 3

    def test_has_multiple_commands(self):
        self.assertTrue(self.pane.has_multiple_commands)

    def test_tmux_window_and_pane_target(self):
        self.assertEqual(self.pane.tmux_window_and_pane_target, 'sample:0.3')


class TestPaneLast(PaneTestCase):
    pane_idx = 3

    def test_is_last(self):
        self.assertTrue(self.pane.is_last)


import mock

from jinja2 import Template
from pymuxinator import exceptions
from pymuxinator.project import Project
from pymuxinator.window import Window
from tests import FixturedTestCase

class TestSampleProject(FixturedTestCase):
    def setUp(self):
        self.project = Project.from_name('sample')

    def test_has_windows(self):
        self.assertTrue(self.project.has_windows)

    def test_windows(self):
        windows = self.project.windows
        self.assertEqual(len(windows), 9)

        for idx, window in enumerate(windows):
            self.assertTrue(isinstance(window, Window))
            self.assertEqual(window.index, idx)
            self.assertTrue(window.config is not None)
            self.assertTrue(window.project_config, self.project.window_config)

    @mock.patch('pymuxinator.project.Project.base_index', return_value=44, new_callable=mock.PropertyMock)
    @mock.patch('pymuxinator.project.Project.name', return_value='name', new_callable=mock.PropertyMock)
    @mock.patch('pymuxinator.project.Project.pre_window', return_value='pre', new_callable=mock.PropertyMock)
    @mock.patch('pymuxinator.project.Project.root', return_value='root', new_callable=mock.PropertyMock)
    @mock.patch('pymuxinator.project.Project.tmux', return_value='ftmux', new_callable=mock.PropertyMock)
    def test_window_config(self, mock_tmux, mock_root, mock_pre, mock_name, mock_base):
        self.assertEqual(self.project.window_config, {
            'name': mock_name.return_value,
            'base_index': mock_base.return_value,
            'pre_window': mock_pre.return_value,
            'root': mock_root.return_value,
            'tmux': mock_tmux.return_value,
        })

    def test_project_name(self):
        self.assertEqual(self.project.name, 'sample')

    @mock.patch('pymuxinator.utils.expand_path', return_value='/home/foo/bar')
    def test_root(self, mock_expand):
        self.assertEqual(self.project.root, '/home/foo/bar')

    def test_pre_window(self):
        self.assertEqual(self.project.pre_window, 'source env/bin/activate')

    @mock.patch('pymuxinator.config.load_template')
    def test_render(self, mock_load_template):
        mock_load_template.return_value = Template('Name: {{ project.name }}')
        self.assertEqual(self.project.render(), 'Name: sample')

    def test_environment(self):
        self.assertEqual(self.project.environment, {
            'FOO': 'bar',
            'BAR': None,
        })

    def test_tmux_command(self):
        self.assertEqual(self.project.tmux_command, 'tmux')

    def test_socket_name(self):
        self.assertEqual(self.project.socket_name, 'foo')

    def test_pre(self):
        self.assertEqual(self.project.pre, 'supervisorctl status my_service')

    def test_tmux_options(self):
        self.assertEqual(self.project.tmux_options, '-f ~/.tmux.testing.conf')

    def test_socket(self):
        self.assertEqual(self.project.socket, '-L foo')

    def test_tmux(self):
        expected = 'tmux -f ~/.tmux.testing.conf -L foo'
        self.assertEqual(self.project.tmux, expected)

    def test_window(self):
        self.assertEqual(self.project.window(1), 'sample:1')
        self.assertEqual(self.project.window('10'), 'sample:10')

    def test_show_tmux_options(self):
        expected = 'tmux -f ~/.tmux.testing.conf -L foo start-server\\; show-option -g'
        self.assertEqual(self.project.show_tmux_options, expected)

    def test_tmux_config(self):
        self.assertTrue(isinstance(self.project.tmux_config, dict))

    @mock.patch('pymuxinator.utils.execute_cmd', return_value='')
    def test_tmux_config_cached(self, mock_call):
        """Test that it doesn't shell out when called more than once"""
        self.project.tmux_config = None

        config = self.project.tmux_config
        config = self.project.tmux_config
        self.assertEqual(mock_call.call_count, 1)

    def test_tmux_config_setter(self):
        config = {'foo': 'bar'}
        self.project.tmux_config = config
        self.assertEqual(self.project.tmux_config, config)

    def test_tmux_base_index(self):
        self.project.tmux_config = {'base-index': 55}
        self.assertEqual(self.project.tmux_base_index, 55)

    def test_tmux_pane_base_index(self):
        self.project.tmux_config = {'pane-base-index': 22}
        self.assertEqual(self.project.tmux_pane_base_index, 22)

    def test_base_index(self):
        self.project.tmux_config = {'base-index': '55'}
        self.assertEqual(self.project.base_index, 55)

    def test_base_index_pane(self):
        self.project.tmux_config = {'pane-base-index': '223'}
        self.assertEqual(self.project.base_index, 223)


class TestCustomTmux(FixturedTestCase):
    def setUp(self):
        self.project = Project.from_name('sample_custom')

    def test_tmux_command(self):
        self.assertEqual(self.project.tmux_command, 'custom')

    def test_pre(self):
        expected = 'foo; bar; baz'
        self.assertEqual(self.project.pre, expected)

    def test_tmux_options(self):
        self.assertEqual(self.project.tmux_options, '')

    def test_socket_path(self):
        self.assertEqual(self.project.socket_path, '/foo/bar')

    def test_socket(self):
        self.assertEqual(self.project.socket, '-S /foo/bar')

    def test_tmux(self):
        expected = 'custom -S /foo/bar'
        self.assertEqual(self.project.tmux, expected)

    def test_show_tmux_options(self):
        expected = 'custom -S /foo/bar start-server\\; show-option -g'
        self.assertEqual(self.project.show_tmux_options, expected)


class TestDefaultSocket(FixturedTestCase):
    def setUp(self):
        self.project = Project.from_name('sample_default_socket')

    def test_tmux(self):
        expected = 'tmux -f ~/.tmux.testing.conf'
        self.assertEqual(self.project.tmux, expected)


class TestValidations(FixturedTestCase):
    def test_no_windows(self):
        with self.assertRaises(exceptions.NoWindowsError):
            project = Project.from_name('windowless')

    def test_invalid_name(self):
        with self.assertRaises(exceptions.NoNameError):
            project = Project.from_name('invalid_name')



from setuptools import setup, find_packages

setup(
    name = "pymuxinator",
    version = "0.0.6",
    author = "Caleb Mingle",
    author_email = "caleb@mingle.cm",
    description = "Tmux session manager",
    url = "http://mingle.cm",
    packages = find_packages(),
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'pymuxinator = pymuxinator.cli:main',
            'mux = pymuxinator.cli:start',
        ],
    },
    tests_require=[
        "nose>=1.0",
        "mock>=1.0",
    ],
    install_requires=[
        "pyyaml>=1.0",
        "Jinja2>=1.0",
    ],
    test_suite = "nose.collector"
)



from pymuxinator import config, exceptions, utils
from pymuxinator.window import Window

class Project(object):
    @classmethod
    def from_name(cls, project_name):
        return cls(config.load(project_name))

    def __init__(self, config):
        # caches
        self._tmux_config = None
        self._windows = []
        self._window_config = None

        self.config = config
        self._validate()

    @property
    def base_index(self):
        base_index = self.tmux_pane_base_index or self.tmux_base_index

        if base_index is not None:
            base_index = int(base_index)

        return base_index

    @property
    def has_windows(self):
        return bool(self._get_config('windows'))

    @property
    def windows(self):
        if not self._windows:
            windows = self._get_config('windows')
            window_config = self.window_config

            if windows:
                self._windows = [Window(window, window_config, idx) for idx, window in enumerate(windows)]

        return self._windows

    @property
    def window_config(self):
        if self._window_config is None:
            self._window_config = {
                'name': self.name,
                'base_index': self.base_index,
                'pre_window': self.pre_window,
                'tmux': self.tmux,
                'root': self.root,
            }

        return self._window_config

    @property
    def name(self):
        name = self._get_config('name')
        return name and utils.shell_escape(name)

    @property
    def root(self):
        root = self._get_config('root')
        return root and utils.shell_escape(utils.expand_path(root))

    @property
    def pre(self):
        pre = self._get_config('pre')

        if isinstance(pre, list):
            pre = '; '.join(pre)

        return pre

    @property
    def pre_window(self):
        return self._get_config('pre_window')

    @property
    def environment(self):
        return self._get_config('environment', {})

    @property
    def tmux_command(self):
        return self._get_config('tmux_command', 'tmux')

    @property
    def tmux_options(self):
        return self._get_config('tmux_options', '').strip()

    @property
    def tmux(self):
        cmd_parts = [self.tmux_command]

        if self.tmux_options:
            cmd_parts.append(self.tmux_options)

        if self.socket:
            cmd_parts.append(self.socket)

        return u' '.join(cmd_parts)

    @property
    def socket_name(self):
        return self._get_config('socket_name')

    @property
    def socket_path(self):
        return self._get_config('socket_path')

    @property
    def socket(self):
        if self.socket_path:
            return u'-S %s' % self.socket_path

        if self.socket_name:
            return u'-L %s' % self.socket_name

    @property
    def show_tmux_options(self):
        return u"%s start-server\\; show-option -g" % self.tmux

    @property
    def tmux_config(self):
        if self._tmux_config is None:
            config = utils.execute_cmd(self.show_tmux_options)
            config = config.split('\n')

            options = {}

            for line in config:
                if line:
                    key, value = line.split(' ', 1)
                    options[key] = value

            self._tmux_config = options

        return self._tmux_config

    @tmux_config.setter
    def tmux_config(self, value):
        self._tmux_config = value

    @property
    def tmux_base_index(self):
        return self.tmux_config.get('base-index')

    @property
    def tmux_pane_base_index(self):
        return self.tmux_config.get('pane-base-index')

    def render(self):
        template = config.load_template('tmux')

        return template.render(
            shell=config.get_env_var('SHELL'),
            project=self,
        )

    def window(self, i):
        return u'%s:%s' % (self.name, str(i))

    def _validate(self):
        if not self.has_windows:
            raise exceptions.NoWindowsError

        if not self.name:
            raise exceptions.NoNameError

    def _get_config(self, key, default=None):
        return self.config.get(key, default)



import argparse
import sys
import os

import pymuxinator.config as config
from pymuxinator.project import Project
from pymuxinator import exceptions, utils


def main():
    CLI(args=sys.argv[1:])


def start():
    args = sys.argv[1:]

    # if it's just `mux api` we want `mux start api`
    if len(args) >= 1:
        args = ['start'] + args

    CLI(args=args)


class CLI(object):
    valid_commands = (
        ('commands', 'list commands',),
        ('start', 'start project',),
        ('list', 'list all projects',),
    )

    def __init__(self, args):
        parser = argparse.ArgumentParser(description='Pymuxinator')
        sub_parsers = parser.add_subparsers(title='commands', description='valid commands')

        for command, help_text in self.valid_commands:
            if hasattr(self, command):
                sub_parser = sub_parsers.add_parser(command, help=help_text or '')

                # add arguments to sub_parser
                if hasattr(self, '_' + command):
                    getattr(self, '_' + command)(sub_parser)

                # set default function for sub_parser
                sub_parser.set_defaults(func=getattr(self, command))

        if len(sys.argv) <= 1:
            parser.print_help()
        else:
            args = parser.parse_args(args)
            args.func(args)

    def _start(self, parser):
        parser.add_argument('project', help='pymuxinator project to start')
        parser.add_argument('--preview', dest='preview', action='store_true', help='preview tmux commands')

    def start(self, args):
        project_name = args.project
        project = None

        try:
            project = Project.from_name(project_name)
        except exceptions.ProjectNotFoundError:
            print u"Project `%s` does not exist" % project_name
        except exceptions.ConfigParseError as e:
            print u"Project `%s` config could not be parsed (%s)" % (project_name, e.message)
        except exceptions.NoWindowsError:
            print u"Project `%s` contains no windows. Please include some windows." % project_name
        except exceptions.NoNameError:
            print u"Project `%s` didn't specify a `project_name`." % project_name
        else:
            commands = project.render()

            if args.preview:
                print commands
            else:
                utils.execute_cmd(commands)

    def commands(self, args):
        commands = list(self.valid_commands)
        commands = sorted(commands, key=lambda x: x[0])

        for command, help_text in commands:
            print command

    def list(self, args):
        configs = config.list_all()

        for config_file in configs:
            base = os.path.basename(config_file)
            print os.path.splitext(base)[0]

